@@ -7,14 +7,14 @@ package AnyEvent::Redis::RipeRedis;
 
 use base qw( Exporter );
 
-our $VERSION = '1.44';
+our $VERSION = '1.46';
 
 use AnyEvent;
 use AnyEvent::Handle;
 use Encode qw( find_encoding is_utf8 );
 use Scalar::Util qw( looks_like_number weaken );
 use Digest::SHA qw( sha1_hex );
-use Carp qw( confess );
+use Carp qw( croak );
 
 BEGIN {
   our @EXPORT_OK = qw(
@@ -37,6 +37,7 @@ BEGIN {
     E_NO_AUTH
     E_WRONG_TYPE
     E_NO_REPLICAS
+    E_BUSY_KEY
   );
 
   our %EXPORT_TAGS = (
@@ -70,6 +71,7 @@ use constant {
   E_NO_AUTH                    => 19,
   E_WRONG_TYPE                 => 20,
   E_NO_REPLICAS                => 21,
+  E_BUSY_KEY                   => 22,
 
   # Operation status
   S_NEED_PERFORM => 1,
@@ -90,7 +92,8 @@ my %SUBUNSUB_CMDS = (
   unsubscribe  => 1,
   punsubscribe => 1,
 );
-my %NEED_SPECPROC = (
+
+my %NEED_POSTPROCESS = (
   %SUBUNSUB_CMDS,
   info   => 1,
   select => 1,
@@ -114,6 +117,7 @@ my %ERR_PREFS_MAP = (
   NOAUTH     => E_NO_AUTH,
   WRONGTYPE  => E_WRONG_TYPE,
   NOREPLICAS => E_NO_REPLICAS,
+  BUSYKEY    => E_BUSY_KEY,
 );
 
 my %EVAL_CACHE;
@@ -122,51 +126,42 @@ my %EVAL_CACHE;
 # Constructor
 sub new {
   my $proto  = shift;
-  my $params = { @_ };
+  my %params = @_;
 
   my $self = ref( $proto ) ? $proto : bless {}, $proto;
 
-  $self->{host}     = $params->{host} || D_HOST;
-  $self->{port}     = $params->{port} || D_PORT;
-  $self->{password} = $params->{password};
-  unless ( defined $params->{database} ) {
-    $params->{database} = D_DB_INDEX;
-  }
-  $self->{database} = $params->{database};
-  $self->encoding( $params->{encoding} );
-  $self->connection_timeout( $params->{connection_timeout} );
-  $self->read_timeout( $params->{read_timeout} );
-  unless ( exists $params->{reconnect} ) {
-    $params->{reconnect} = 1;
-  }
-  $self->{reconnect} = $params->{reconnect};
+  $self->{host}      = $params{host} || D_HOST;
+  $self->{port}      = $params{port} || D_PORT;
+  $self->{password}  = $params{password};
+  $self->{database}  = defined $params{database} ? $params{database} : D_DB_INDEX;
+  $self->{reconnect} = exists $params{reconnect} ? $params{reconnect} : 1;
+  $self->{on_connect}       = $params{on_connect};
+  $self->{on_disconnect}    = $params{on_disconnect};
+  $self->{on_connect_error} = $params{on_connect_error};
 
-  my $handle_params = $params->{handle_params};
-  if ( !defined $params->{handle_params} ) {
-    $handle_params = {};
-  }
-  foreach my $p_name ( qw( linger autocork ) ) {
-    if ( !exists $handle_params->{ $p_name } && defined $params->{ $p_name } ) {
-      $handle_params->{ $p_name } = $params->{ $p_name };
+  $self->encoding( $params{encoding} );
+  $self->connection_timeout( $params{connection_timeout} );
+  $self->read_timeout( $params{read_timeout} );
+  $self->on_error( $params{on_error} );
+
+  my $hdl_params = $params{handle_params} || {};
+  foreach my $name ( qw( linger autocork ) ) {
+    if ( !exists $hdl_params->{$name} && defined $params{$name} ) {
+      $hdl_params->{$name} = $params{$name};
     }
   }
-  $self->{handle_params} = $handle_params;
-
-  $self->{on_connect}       = $params->{on_connect};
-  $self->{on_disconnect}    = $params->{on_disconnect};
-  $self->{on_connect_error} = $params->{on_connect_error};
-  $self->on_error( $params->{on_error} );
+  $self->{handle_params} = $hdl_params;
 
   $self->{_handle}         = undef;
   $self->{_connected}      = 0;
-  $self->{_lazy_conn_st}   = $params->{lazy};
+  $self->{_lazy_conn_st}   = $params{lazy};
   $self->{_auth_st}        = S_NEED_PERFORM;
   $self->{_select_db_st}   = S_NEED_PERFORM;
   $self->{_ready_to_write} = 0;
   $self->{_input_queue}    = [];
   $self->{_tmp_queue}      = [];
   $self->{_process_queue}  = [];
-  $self->{_multi_lock}     = 0;
+  $self->{_txn_lock}       = 0;
   $self->{_subs}           = {};
   $self->{_subs_num}       = 0;
 
@@ -180,11 +175,9 @@ sub new {
 ####
 sub multi {
   my $self = shift;
+  my $cmd  = $self->_prepare_cmd( 'multi', [ @_ ] );
 
-  my $cmd = $self->_prepare_cmd( 'multi', [ @_ ] );
-
-  $self->{_multi_lock} = 1;
-
+  $self->{_txn_lock} = 1;
   $self->_execute_cmd( $cmd );
 
   return;
@@ -193,65 +186,18 @@ sub multi {
 ####
 sub exec {
   my $self = shift;
+  my $cmd  = $self->_prepare_cmd( 'exec', [ @_ ] );
 
-  my $cmd = $self->_prepare_cmd( 'exec', [ @_ ] );
-
-  $self->{_multi_lock} = 0;
-
+  $self->{_txn_lock} = 0;
   $self->_execute_cmd( $cmd );
 
   return;
 }
 
-####
-sub subscribe {
-  my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'subscribe', [ @_ ] );
-
-  $self->_subunsub( $cmd );
-
-  return;
-}
-
-####
-sub psubscribe {
-  my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'psubscribe', [ @_ ] );
-
-  $self->_subunsub( $cmd );
-
-  return;
-}
-
-####
-sub unsubscribe {
-  my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'unsubscribe', [ @_ ] );
-
-  $self->_subunsub( $cmd );
-
-  return;
-}
-
-####
-sub punsubscribe {
-  my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'punsubscribe', [ @_ ] );
-
-  $self->_subunsub( $cmd );
-
-  return;
-}
-
 ####
 sub eval_cached {
   my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'evalsha', [ @_ ] );
+  my $cmd  = $self->_prepare_cmd( 'evalsha', [ @_ ] );
 
   $cmd->{script} = $cmd->{args}[0];
   unless ( exists $EVAL_CACHE{ $cmd->{script} } ) {
@@ -282,8 +228,9 @@ sub encoding {
 
     if ( defined $enc ) {
       $self->{encoding} = find_encoding( $enc );
+
       unless ( defined $self->{encoding} ) {
-        confess "Encoding '$enc' not found";
+        croak "Encoding '$enc' not found";
       }
     }
     else {
@@ -323,43 +270,76 @@ sub selected_database {
   return $self->{database};
 }
 
-# Create more accessors
+# Generate additional methods and accessors
 {
   no strict 'refs';
 
-  foreach my $attr_pref ( qw( connection read ) ) {
-    my $attr_name = $attr_pref . '_timeout';
+  foreach my $kwd ( keys %SUBUNSUB_CMDS ) {
+    *{$kwd} = sub {
+      my $self = shift;
+      my $cmd  = $self->_prepare_cmd( $kwd, [ @_ ] );
 
-    *{$attr_name} = sub {
+      if ( exists $SUB_CMDS{ $cmd->{kwd} } && !defined $cmd->{on_message} ) {
+        croak "'on_message' callback must be specified";
+      }
+
+      if ( $self->{_txn_lock} ) {
+        AE::postpone(
+          sub {
+            $self->_process_cmd_error( $cmd,
+                "Command '$cmd->{kwd}' not allowed after 'multi' command."
+                    . ' First, the transaction must be finalized.',
+                E_OPRN_ERROR );
+          }
+        );
+
+        return;
+      }
+
+      $cmd->{replies_cnt} = scalar @{ $cmd->{args} };
+
+      if ( defined $cmd->{on_done} ) {
+        my $on_done = $cmd->{on_done};
+
+        $cmd->{on_done} = sub {
+          $on_done->( @{ $_[0] } );
+        }
+      }
+
+      $self->_execute_cmd( $cmd );
+
+      return;
+    },
+  }
+
+  foreach my $name_pref ( qw( connection read ) ) {
+    my $name = $name_pref . '_timeout';
+
+    *{$name} = sub {
       my $self = shift;
 
       if ( @_ ) {
         my $timeout = shift;
 
-        if (
-          defined $timeout
-            && ( !looks_like_number( $timeout ) || $timeout < 0 )
-            ) {
-          confess ucfirst( $attr_pref ) . ' timeout must be a positive number';
+        if ( defined $timeout && ( !looks_like_number( $timeout ) || $timeout < 0 ) ) {
+          croak ucfirst( $name_pref ) . ' timeout must be a positive number';
         }
-        $self->{ $attr_name } = $timeout;
+        $self->{$name} = $timeout;
       }
 
-      return $self->{ $attr_name };
+      return $self->{$name};
     }
   }
 
-  foreach my $attr_name (
-    qw( reconnect on_connect on_disconnect on_connect_error )
-      ) {
-    *{$attr_name} = sub {
+  foreach my $name ( qw( reconnect on_connect on_disconnect on_connect_error ) ) {
+    *{$name} = sub {
       my $self = shift;
 
       if ( @_ ) {
-        $self->{ $attr_name } = shift;
+        $self->{$name} = shift;
       }
 
-      return $self->{ $attr_name };
+      return $self->{$name};
     }
   }
 }
@@ -452,7 +432,7 @@ sub _get_on_rtimeout {
   weaken( $self );
 
   return sub {
-    if ( @{$self->{_process_queue}} ) {
+    if ( @{ $self->{_process_queue} } ) {
       $self->_disconnect( 'Read timed out.', E_READ_TIMEDOUT );
     }
     else {
@@ -543,9 +523,9 @@ sub _get_on_read {
           elsif ( $type eq '*' ) {
             if ( $data > 0 ) {
               push( @bufs,
-                { data        => [],
-                  err_code    => undef,
-                  chunks_left => $data,
+                { data       => [],
+                  err_code   => undef,
+                  chunks_cnt => $data,
                 }
               );
               $bufs_num++;
@@ -562,8 +542,8 @@ sub _get_on_read {
           elsif ( $type eq '-' ) {
             $err_code = E_OPRN_ERROR;
             if ( $data =~ m/^([A-Z]{3,}) / ) {
-              if ( exists $ERR_PREFS_MAP{ $1 } ) {
-                $err_code = $ERR_PREFS_MAP{ $1 };
+              if ( exists $ERR_PREFS_MAP{$1} ) {
+                $err_code = $ERR_PREFS_MAP{$1};
               }
             }
           }
@@ -584,8 +564,8 @@ sub _get_on_read {
           }
           $curr_buf->{err_code} = E_OPRN_ERROR;
         }
-        push( @{$curr_buf->{data}}, $data );
-        if ( --$curr_buf->{chunks_left} > 0 ) {
+        push( @{ $curr_buf->{data} }, $data );
+        if ( --$curr_buf->{chunks_cnt} > 0 ) {
           next MAIN;
         }
 
@@ -610,16 +590,16 @@ sub _prepare_cmd {
 
   my $cmd;
   if ( ref( $args->[-1] ) eq 'HASH' ) {
-    $cmd = pop( @{$args} );
+    $cmd = pop @{$args};
   }
   else {
     $cmd = {};
     if ( ref( $args->[-1] ) eq 'CODE' ) {
-      if ( exists $SUB_CMDS{ $kwd } ) {
-        $cmd->{on_message} = pop( @{$args} );
+      if ( exists $SUB_CMDS{$kwd} ) {
+        $cmd->{on_message} = pop @{$args};
       }
       else {
-        $cmd->{on_reply} = pop( @{$args} );
+        $cmd->{on_reply} = pop @{$args};
       }
     }
   }
@@ -629,43 +609,6 @@ sub _prepare_cmd {
   return $cmd;
 }
 
-####
-sub _subunsub {
-  my $self = shift;
-  my $cmd  = shift;
-
-  if ( exists $SUB_CMDS{ $cmd->{kwd} } && !defined $cmd->{on_message} ) {
-    confess "'on_message' callback must be specified";
-  }
-
-  if ( $self->{_multi_lock} ) {
-    AE::postpone(
-      sub {
-        $self->_process_cmd_error( $cmd,
-            "Command '$cmd->{kwd}' not allowed after 'multi' command."
-                . ' First, the transaction must be finalized.',
-            E_OPRN_ERROR );
-      }
-    );
-
-    return;
-  }
-
-  $cmd->{repls_left} = scalar( @{$cmd->{args}} );
-
-  if ( defined $cmd->{on_done} ) {
-    my $on_done = $cmd->{on_done};
-
-    $cmd->{on_done} = sub {
-      $on_done->( @{$_[0]} );
-    }
-  }
-
-  $self->_execute_cmd( $cmd );
-
-  return;
-}
-
 ####
 sub _execute_cmd {
   my $self = shift;
@@ -694,16 +637,15 @@ sub _execute_cmd {
     else {
       AE::postpone(
         sub {
-          $self->_process_cmd_error( $cmd,
-              "Operation '$cmd->{kwd}' aborted: No connection to the server.",
-              E_NO_CONN );
+          $self->_process_cmd_error( $cmd, "Operation '$cmd->{kwd}' aborted:"
+              . ' No connection to the server.', E_NO_CONN );
         }
       );
 
       return;
     }
 
-    push( @{$self->{_input_queue}}, $cmd );
+    push( @{ $self->{_input_queue} }, $cmd );
 
     return;
   }
@@ -719,7 +661,7 @@ sub _push_write {
   my $cmd  = shift;
 
   my $cmd_str = '';
-  foreach my $token ( $cmd->{kwd}, @{$cmd->{args}} ) {
+  foreach my $token ( $cmd->{kwd}, @{ $cmd->{args} } ) {
     unless ( defined $token ) {
       $token = '';
     }
@@ -728,14 +670,15 @@ sub _push_write {
     }
     $cmd_str .= '$' . length( $token ) . EOL . $token . EOL;
   }
-  $cmd_str = '*' . ( scalar( @{$cmd->{args}} ) + 1 ) . EOL . $cmd_str;
+  $cmd_str = '*' . ( scalar( @{ $cmd->{args} } ) + 1 ) . EOL . $cmd_str;
 
   my $handle = $self->{_handle};
-  if ( defined $self->{read_timeout} && !@{$self->{_process_queue}} ) {
+  if ( defined $self->{read_timeout} && !@{ $self->{_process_queue} } ) {
     $handle->rtimeout_reset();
     $handle->rtimeout( $self->{read_timeout} );
   }
-  push( @{$self->{_process_queue}}, $cmd );
+  push( @{ $self->{_process_queue} }, $cmd );
+
   $handle->push_write( $cmd_str );
 
   return;
@@ -788,7 +731,7 @@ sub _select_db {
       args => [ $self->{database} ],
 
       on_done => sub {
-        $self->{_select_db_st} = S_IS_DONE;
+        $self->{_select_db_st}   = S_IS_DONE;
         $self->{_ready_to_write} = 1;
         $self->_flush_input_queue();
       },
@@ -807,9 +750,10 @@ sub _select_db {
 sub _flush_input_queue {
   my $self = shift;
 
-  $self->{_tmp_queue} = $self->{_input_queue};
+  $self->{_tmp_queue}   = $self->{_input_queue};
   $self->{_input_queue} = [];
-  while ( my $cmd = shift( @{$self->{_tmp_queue}} ) ) {
+
+  while ( my $cmd = shift @{ $self->{_tmp_queue} } ) {
     $self->_push_write( $cmd );
   }
 
@@ -818,54 +762,50 @@ sub _flush_input_queue {
 
 ####
 sub _process_reply {
-  my $self = shift;
+  my $self     = shift;
+  my $data     = shift;
+  my $err_code = shift;
 
-  if ( defined $_[1] ) {
-    my $cmd = shift( @{$self->{_process_queue}} );
+  if ( defined $err_code ) {
+    my $cmd = shift @{ $self->{_process_queue} };
 
     unless ( defined $cmd ) {
-      $self->_disconnect( "Don't known how process error."
+      $self->_disconnect( "Don't know how process error message."
           . ' Processing queue is empty.', E_UNEXPECTED_DATA );
 
       return;
     }
 
-    $self->_process_cmd_error( $cmd, ref( $_[0] )
-        ? ( "Operation '$cmd->{kwd}' completed with errors.", @_[ 1, 0 ] )
-        : @_ );
+    $self->_process_cmd_error( $cmd, ref( $data )
+        ? ( "Operation '$cmd->{kwd}' completed with errors.", $err_code, $data )
+        : $data, $err_code );
   }
-  elsif (
-    $self->{_subs_num} > 0 && ref( $_[0] )
-      && exists $MSG_TYPES{ $_[0][0] }
-      ) {
-    my $msg    = $_[0];
-    my $on_msg = $self->{_subs}{ $msg->[1] };
+  elsif ( $self->{_subs_num} > 0 && ref( $data ) && exists $MSG_TYPES{ $data->[0] } ) {
+    my $on_msg = $self->{_subs}{ $data->[1] };
 
     unless ( defined $on_msg ) {
-      $self->_disconnect( "Don't known how process published message."
-          . " Unknown channel or pattern '$msg->[1]'.", E_UNEXPECTED_DATA );
+      $self->_disconnect( "Don't know how process published message."
+          . " Unknown channel or pattern '$data->[1]'.", E_UNEXPECTED_DATA );
 
       return;
     }
 
-    $on_msg->( $msg->[0] eq 'pmessage' ? @{$msg}[ 2, 3, 1 ]
-        : @{$msg}[ 1, 2 ] );
+    $on_msg->( $data->[0] eq 'pmessage' ? @{$data}[ 2, 3, 1 ] : @{$data}[ 1, 2 ] );
   }
   else {
     my $cmd = $self->{_process_queue}[0];
 
     unless ( defined $cmd ) {
-      $self->_disconnect( "Don't known how process reply."
+      $self->_disconnect( "Don't know how process reply."
           . ' Processing queue is empty.', E_UNEXPECTED_DATA );
 
       return;
     }
 
-    if ( !defined $cmd->{repls_left} || --$cmd->{repls_left} <= 0 ) {
-      shift( @{$self->{_process_queue}} );
+    if ( !defined $cmd->{replies_cnt} || --$cmd->{replies_cnt} <= 0 ) {
+      shift @{ $self->{_process_queue} };
     }
-
-    $self->_process_cmd_success( $cmd, $_[0] );
+    $self->_process_cmd_success( $cmd, $data );
   }
 
   return;
@@ -878,7 +818,7 @@ sub _process_cmd_error {
 
   if ( $_[1] == E_NO_SCRIPT && defined $cmd->{script} ) {
     $cmd->{kwd}     = 'eval';
-    $cmd->{args}[0] = $cmd->{script};
+    $cmd->{args}[0] = delete $cmd->{script};
 
     $self->_push_write( $cmd );
 
@@ -904,17 +844,17 @@ sub _process_cmd_success {
   my $cmd  = shift;
   my $data = shift;
 
-  if ( exists $NEED_SPECPROC{ $cmd->{kwd} } ) {
+  if ( exists $NEED_POSTPROCESS{ $cmd->{kwd} } ) {
     my $kwd = $cmd->{kwd};
 
-    if ( exists $SUBUNSUB_CMDS{ $kwd } ) {
-      shift( @{$data} );
+    if ( exists $SUBUNSUB_CMDS{$kwd} ) {
+      shift @{$data};
 
-      if ( exists $SUB_CMDS{ $kwd } ) {
+      if ( exists $SUB_CMDS{$kwd} ) {
         $self->{_subs}{ $data->[0] } = $cmd->{on_message};
       }
       else { # unsubscribe or punsubscribe
-        delete( $self->{_subs}{ $data->[0] } );
+        delete $self->{_subs}{ $data->[0] };
       }
 
       $self->{_subs_num} = $data->[1];
@@ -942,8 +882,10 @@ sub _process_cmd_success {
 
 ####
 sub _parse_info {
-  return { map { split( m/:/, $_, 2 ) } grep( m/^[^#]/ ,
-      split( EOL, $_[1] ) ) };
+  return {
+    map { split( m/:/, $_, 2 ) }
+    grep { m/^[^#]/ } split( EOL, $_[1] )
+  };
 }
 
 ####
@@ -960,7 +902,7 @@ sub _disconnect {
   $self->{_auth_st}        = S_NEED_PERFORM;
   $self->{_select_db_st}   = S_NEED_PERFORM;
   $self->{_ready_to_write} = 0;
-  $self->{_multi_lock}     = 0;
+  $self->{_txn_lock}       = 0;
 
   $self->_abort_all( @_ );
 
@@ -978,9 +920,9 @@ sub _abort_all {
   my $err_code = shift;
 
   my @unfin_cmds = (
-    @{$self->{_process_queue}},
-    @{$self->{_tmp_queue}},
-    @{$self->{_input_queue}},
+    @{ $self->{_process_queue} },
+    @{ $self->{_tmp_queue} },
+    @{ $self->{_input_queue} },
   );
 
   $self->{_input_queue}   = [];
@@ -1002,8 +944,8 @@ sub _abort_all {
     }
 
     foreach my $cmd ( @unfin_cmds ) {
-      $self->_process_cmd_error( $cmd,
-          "Operation '$cmd->{kwd}' aborted: $err_msg", $err_code );
+      $self->_process_cmd_error( $cmd, "Operation '$cmd->{kwd}' aborted: "
+          . $err_msg, $err_code );
     }
   }
 
@@ -1019,8 +961,7 @@ sub AUTOLOAD {
 
   my $sub = sub {
     my $self = shift;
-
-    my $cmd = $self->_prepare_cmd( $kwd, [ @_ ] );
+    my $cmd  = $self->_prepare_cmd( $kwd, [ @_ ] );
 
     $self->_execute_cmd( $cmd );
 
@@ -1041,9 +982,9 @@ sub DESTROY {
 
   if ( defined $self->{_handle} ) {
     my @unfin_cmds = (
-      @{$self->{_process_queue}},
-      @{$self->{_tmp_queue}},
-      @{$self->{_input_queue}},
+      @{ $self->{_process_queue} },
+      @{ $self->{_tmp_queue} },
+      @{ $self->{_input_queue} },
     );
 
     foreach my $cmd ( @unfin_cmds ) {
@@ -1111,7 +1052,7 @@ feature
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1152,9 +1093,11 @@ feature
 
   $redis->quit(
     sub {
-      if ( defined $_[1] ) {
-        my $err_msg  = $_[1];
-        my $err_code = $_[2];
+      shift;
+
+      if ( @_ ) {
+        my $err_msg  = shift;
+        my $err_code = shift;
 
         warn "[$err_code] $err_msg\n";
       }
@@ -1359,7 +1302,7 @@ The full list of the Redis commands can be found here: L<http://redis.io/command
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1376,7 +1319,7 @@ The full list of the Redis commands can be found here: L<http://redis.io/command
     { on_reply => sub {
         my $data = shift;
 
-        if ( defined $_[0] ) {
+        if ( @_ ) {
           my $err_msg  = shift;
           my $err_code = shift;
 
@@ -1480,19 +1423,25 @@ error message and C<code()> to get error code.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
-        foreach my $chunk ( @{$data} ) {
-          if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
-            my $oprn_err_msg  = $chunk->message();
-            my $oprn_err_code = $chunk->code();
+        if ( defined $data ) {
+          foreach my $chunk ( @{$data} ) {
+            if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
+              my $oprn_err_msg  = $chunk->message();
+              my $oprn_err_code = $chunk->code();
 
-            # error handling...
+              # error handling...
+            }
           }
+
+          return;
         }
 
+        # error handling...
+
         return;
       }
 
@@ -1516,14 +1465,18 @@ error message and C<code()> to get error code.
         my $err_code = shift;
         my $data     = shift;
 
-        foreach my $chunk ( @{$data} ) {
-          if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
-            my $oprn_err_msg  = $chunk->message();
-            my $oprn_err_code = $chunk->code();
+        if ( defined $data ) {
+          foreach my $chunk ( @{$data} ) {
+            if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
+              my $oprn_err_msg  = $chunk->message();
+              my $oprn_err_code = $chunk->code();
 
-            # error handling...
+              # error handling...
+            }
           }
         }
+
+        # error handling...
       },
     }
   );
@@ -1560,7 +1513,7 @@ and C<PUNSUBSCRIBE> commands.
     { on_reply => sub {
         my $data = shift;
 
-        if ( defined $_[0] ) {
+        if ( @_ ) {
           my $err_msg  = shift;
           my $err_code = shift;
 
@@ -1650,7 +1603,7 @@ Subscribes the client to the given patterns.
     { on_reply => sub {
         my $data = shift;
 
-        if ( defined $_[0] ) {
+        if ( @_ ) {
           my $err_msg  = shift;
           my $err_code = shift;
 
@@ -1752,7 +1705,7 @@ channel will be sent to the client.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1812,7 +1765,7 @@ pattern will be sent to the client.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1892,7 +1845,7 @@ instead.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1920,19 +1873,23 @@ command.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
-        foreach my $chunk ( @{$data} ) {
-          if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
-            my $nested_err_msg  = $chunk->message();
-            my $nested_err_code = $chunk->code();
+        if ( defined $data ) {
+          foreach my $chunk ( @{$data} ) {
+            if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
+              my $nested_err_msg  = $chunk->message();
+              my $nested_err_code = $chunk->code();
 
-            # error handling...
+              # error handling...
+            }
           }
         }
 
+        # error handling...
+
         return;
       }
 
@@ -1952,14 +1909,18 @@ command.
         my $err_code = shift;
         my $data     = shift;
 
-        foreach my $chunk ( @{$data} ) {
-          if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
-            my $nested_err_msg  = $chunk->message();
-            my $nested_err_code = $chunk->code();
+        if ( defined $data ) {
+          foreach my $chunk ( @{$data} ) {
+            if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
+              my $nested_err_msg  = $chunk->message();
+              my $nested_err_code = $chunk->code();
 
-            # error handling...
+              # error handling...
+            }
           }
         }
+
+        # error handling...
       }
     }
   );
@@ -2081,6 +2042,10 @@ Operation against a key holding the wrong kind of value.
 
 Not enough good slaves to write.
 
+=item E_BUSY_KEY
+
+Target key name already exists.
+
 =back
 
 =head1 DISCONNECTION
@@ -2179,7 +2144,7 @@ Ivan Kruglov
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2012-2014, Eugene Ponizovsky, E<lt>ponizovsky@gmail.comE<gt>.
+Copyright (c) 2012-2015, Eugene Ponizovsky, E<lt>ponizovsky@gmail.comE<gt>.
 All rights reserved.
 
 This module is free software; you can redistribute it and/or modify it under
